%dip_systemdoctor   Checks the DIPlib library installation

% (C) Copyright 1999-2006               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Michael van Ginkel, February 2005.
% 28 Feb 2005  - Also checking the time stamp of the libraries. (CL)
% 17 June 2005 - Added MESSAGE_FROM_DIP__INITIALISE parameter. (MvG)
% 17 June 2005 - LOCATION is now INITMEX, with the name of the MEX-file in it. (CL)
% 2 May 2006   - Rewrote small portions of code, added input arg checks and defaults,
%                removed VERSION input, and modified the output. (CL)

function dip_systemdoctor(initmex,message_from_dip__initialise)

fnddip   = '';
fnddipio = '';
fnddml   = '';
if strcmp(computer,'PCWIN')
   psp = ';';
   dsp = '\';
   soext = '.dll';
   v = version;
   ii = find(v=='.'); if length(ii)>0, v = v(1:ii(1)-1); end % Just the major version number.
   v = eval(v);
   if v>=6
      scanpaths = [matlabroot,'\bin\win32'];
   else
      scanpaths = [matlabroot,'\bin\'];
   end
   envpath=getenv('PATH');
   if ~isempty(envpath)
      scanpaths = [scanpaths,';',envpath];
   end
else
   % Standard Unix settings, ok for Linux and Solaris.
   psp = ':';
   dsp = '/';
   soext = '.so';
   scanpaths = getenv('LD_LIBRARY_PATH');
end
libdip   = ['libdip',  soext];
libdipio = ['libdipio',soext];
libdml   = ['libdml',  soext];
disp(['   The files ',libdip,', ',libdipio,' and ',libdml,' must be available']);
disp('   on the library search path. Looking for them now:');

% Make a cell array of the path elements...
pathix = [0,find(scanpaths==psp),length(scanpaths)+1];
paths = cell(length(pathix)-1,1);
for ii=1:length(paths)
   paths{ii} = scanpaths(pathix(ii)+1:pathix(ii+1)-1);
end
% if isempty(paths) % This never happens!!!

% Now let's scan for the DIP crap
for ii=1:length(paths)
   disp(['     Scanning ',paths{ii}]);
   cfile=[paths{ii},dsp,libdip];
   if exist( cfile, 'file' )
      disp(['        Found ',libdip]);
      if isempty(fnddip)
         fnddip=cfile;
      end
   end
   cfile=[paths{ii},dsp,libdipio];
   if exist( cfile, 'file' )
      disp(['        Found ',libdipio]);
      if isempty(fnddipio)
         fnddipio=cfile;
      end
   end
   cfile=[paths{ii},dsp,libdml];
   if exist( cfile, 'file' )
      disp(['        Found ',libdml]);
      if isempty(fnddml)
         fnddml=cfile;
      end
   end
end
disp(' ');

disp('   Summary of Library locations:');
if fnddip
   dipt = dir(fnddip); dipt = dipt.date;
   disp(['     ',fnddip,' (',dipt,')']);
   dipt = floor(datenum(dipt));
else
   disp(['     ',libdip,' not found']);
   dipt = 0;
end
if fnddipio
   dipiot = dir(fnddipio); dipiot = dipiot.date;
   disp(['     ',fnddipio,' (',dipiot,')']);
   dipiot = floor(datenum(dipiot));
else
   disp(['     ',libdipio,'  not found']);
   dipiot = 0;
end
if fnddml
   dmlt = dir(fnddml); dmlt = dmlt.date;
   disp(['     ',fnddml,' (',dmlt,')']);
   dmlt = floor(datenum(dmlt));
else
   disp(['     ',libdml,' not found']);
   dmlt = 0;
end

if isempty(fnddip) | isempty(fnddipio) | isempty(fnddml)
   disp(' ');
   disp('   Some or all of the relevant libraries were not found.');
else
   if nargin<1
      initmex = which('dip__initialise','in','dip_initialise');
   end
   initt = dir(initmex); initt = initt.date;
   disp(['     ',initmex,' (',initt,')']);
   initt = floor(datenum(initt));
   disp(' ');
   disp('   All the relevant libraries were found.');
   if any([dipt,dipiot,dmlt]~=initt)
      disp(' ');
      disp(['   Since the dates on the found libraries and the toolbox functions   ';...
            '   do not match, it is possible that the wrong version of these       ';...
            '   libraries is being used. Please remove the old DIPimage/DIPlib     ';...
            '   files from your system, and make sure the files found come from the';...
            '   correct DIPimage distribution.                                     '])
   end
   if ~strcmp(computer,'PCWIN')
      disp(' ');
      disp(['   Creating a temporary file to do a ldd scan. This may provide you   ';...
            '   with more information or, in the worst case, more information to   ';...
            '   send to us...                                                      ']);
      disp(' ');
      fd = fopen('/tmp/dip_tryldd','w');
      fprintf(fd,'#! /bin/sh\n\n');
      if length(paths)>0
         fprintf(fd,['LD_LIBRARY_PATH="',paths{1},'"\n']);
      end
      for ii=2:length(paths)
         fprintf(fd,['LD_LIBRARY_PATH="$LD_LIBRARY_PATH:',paths{ii},'"\n']);
      end
      fprintf(fd,'export LD_LIBRARY_PATH\n\n');
      fprintf(fd,['ldd ',initmex,'\n']);
      fclose(fd);
      system('chmod +x /tmp/dip_tryldd');
      system('/tmp/dip_tryldd; exit');
      delete('/tmp/dip_tryldd');
   end
   if nargin>=2 & ~isempty(message_from_dip__initialise)
      disp(' ');
      disp(['   Finally: this is the error message generated by the low-level      ';...
            '   initialisation function:                                           ']);
      disp(['   ',message_from_dip__initialise]);
   end
end
